import stamp.core.*;  //Core software
import stamp.peripheral.adc.*;  //Analogue to Digital Class for accelerometer
import stamp.peripheral.gps.*;  //GPS Class for location and navigation
//import stamp.peripheral.ping.*;  //PING Class - two instances required for front and back


//CarMain.. Always running on the system
public class CarMain {

  //Only have 16 pins available

  /*
  hokay new stuff.. after input from Peter Verkaik 12.05.2008
  You can only have 6 active VP's at any time.
  If you want to use the Timer VP, which really cannot be stopped,
  that leaves you with 5 slots to run different VP's.
  All transmit uarts can occupy one slot, as can all the ADC.
  PWM must of couse keep running.

  1 Timer (always active)
  2 PWM (always active)
  1 Uart receive (always active)
  leaves 2 slots to run either ADC or Uart transmit/receive.
  */

  // Timer (always active, used to update the state vector from sensor input, 0 pins)
  static Timer mainTimer = new Timer(); //first VP

  //placeholders declared but no VP slots occupied yet
  static ADC xAccm2g;
  static ADC yAccm2g;
  static Accelerometer accx;
  static Accelerometer accy;

  static Uart txBaseStation;
  static Uart rxBaseStation;
  static StringBuffer buffer;
  static char commandType;
  static StringBuffer commandValue;
  static char c;
  static String commValue = "";


  static Uart sio;
  static GPS_Parallax mygps;
  static boolean sendsv;

  //wanting to use this to get command.. timer checks if a byte is available
  //5 times a second.. if there is a byte and it is a begin command then process
  static void getCommand()
  {
    c = 0xff; //init char
    commandType = 0xff; //init char
    commandValue.clear();

    //check the ole byte available to see if it is a start command
    if (rxBaseStation.byteAvailable())
    {
      c = (char)rxBaseStation.receiveByte();
      if(c == '{')
      {
        commandType = (char)rxBaseStation.receiveByte(); //get the type of command
        //gather the additional value of the command
        while (c != '}')
        {
           if (rxBaseStation.byteAvailable())
           {
             c = (char)rxBaseStation.receiveByte();
             if(c != '}')
             {
                commandValue.append(c);
             }
           }
        }
        processCommand();
        //Also need to factor in possibility of queued commands..
      }

    }
  }

  static void main()
  {
    //Accelerometer class.. (temp active, 2 / 3 pins)
    xAccm2g = new ADC(CPU.pin9, CPU.pin8); //create ADC
    xAccm2g.stop(); //and stop it to free VP slot
    yAccm2g = new ADC(CPU.pin11, CPU.pin10); //ditto
    yAccm2g.stop();
    accx = new Accelerometer(xAccm2g); //create 2 accelerometer objects
    accy = new Accelerometer(yAccm2g);

    // Uart receive (always active, used to receive infro from base station laptop, 1 pin)
    rxBaseStation = new Uart(Uart.dirReceive, CPU.pin1, Uart.dontInvert, Uart.speed9600, Uart.stop1);
    commandValue = new StringBuffer(128);

    //Uart transmit to send data to base station (temp active, 1 pin)
    txBaseStation = new Uart(Uart.dirTransmit, CPU.pin0, Uart.dontInvert, Uart.speed9600, Uart.stop1);
    txBaseStation.stop();

    //ready to rock and roll
    txBaseStation.start();
    txBaseStation.sendString("Car started awaiting commands: \n\r");
    txBaseStation.stop();

    mainTimer.mark(); //begin the timer
    sendsv = false; //dont send state to laptop unless commanded to start

    while(true)
    {
      //Process this loop 5 times a second unless receive a stop command from laptop
      if (mainTimer.timeout(200))
      {
        if (rxBaseStation.byteAvailable())
        {
          getCommand(); //process inbound bytes into a command
        }

        if(sendsv)
        {
          sendState(); //send the rawdata to the laptop
        }

        mainTimer.mark(); //start the ole timer off again
      }
    }
  }

  //Send the raw data to the laptop (using accel data to test)
  static void sendState()
  {
    txBaseStation.start(); //start the transmit to laptop Uart

    //transmit the accelerometer x and y axis commands to laptop
    txBaseStation.sendString("{1" + Integer.toString(accx.value()) + "}");
    txBaseStation.sendString("{2" + Integer.toString(accy.value()) + "}");

    txBaseStation.stop(); //stop the transmit to laptop Uart
  }

  static void processCommand()
  {
    //command has been gathered.. now do something with it..
    //txBaseStation.start();
    //txBaseStation.sendString(tempCommand.toString());
    //txBaseStation.stop();
    commValue = commandValue.toString();

    switch (commandType)
    {
      case '0': //start sending state to laptop
        if (commValue.equals("true"))
        {
          sendsv = true;
        }
        else
        {
          sendsv = false;
        }
      break;
      case '1': //send drive command
        //engine.update((int)commandValue, 2304):
        txBaseStation.start();
        txBaseStation.sendString("Engine PWM update: " + commValue);
        txBaseStation.stop();
      break;
      case '2': //send steering command
        txBaseStation.start();
        txBaseStation.sendString("Steering PWM update: " + commValue);
        txBaseStation.stop();
      break;
    }
  }
}